/* Altivario Arduino avec capteur de pression MS5611
  Ecran LCD 84 x 48 PCB8544

  Altivario_Parapente_v1.0.0 : reprise du vario modélisme
  Altivario_Parapente_v1.0.5 : début optimisation affichage (1243 ms pour un affichage complet !)
  Altivario_Parapente_v1.0.6 : remplacement RTC DS1307 par RTC DS3231
  Altivario_Parapente_sonore_v1.0.0 : retour à la version sonore uniquement (43% de mémoire programme et 59% de RAM)
  Altivario_Parapente_sonore_v1.0.1 : ajout RTC DS3231
  Altivario_Parapente_v1.0.7 : ajout setup display : affichage unités
  Altivario_Parapente_v1.0.8 : remplacement TFT (trop lent) par OLED 1.3" 128 x 64 SH1106 avec library U8glib.h => boucles d'affichage pour affichage complet en 178ms => trop long, perturbe le vario
  Altivario_Parapente_v1.0.9 : remplacement library U8glib.h par SSD1306Ascii.h => temps affichage OLED = 37ms
  Altivario_Parapente_v1.0.10 : retour Ecran TFT SPI 128x128 1.44p ILI9163 avec MAJ library
  Altivario_Parapente_v1.0.11 : tentative resolution bug affichage 0 en trop sur écran TFT => passage des altitudes de float à int
  Altivario_Parapente_v1.0.12 : TFT pas suffisamment visible au soleil => passage au LCD 84 x 48 PCD8544; correction bug batteryPercentage faux si batteryVoltage < 3.3V
  Altivario_Parapente_v1.0.13 : Ajout bouton poussoir sur D2 pour affichage données min/max du vol
  Altivario_Parapente_v1.0.14 : Suppression bouton poussoir sur D2; affichage données min/max du vol 1x /seconde
  Altivario_Parapente_v1.0.15 : Ajout version firmware au démarrage
  Altivario_Parapente_v1.0.16 : Suppression affichage altitude absolue et tension batterie
*/

#include <Wire.h>
#include "RTClib.h"
RTC_DS3231 rtc;
#include <SPI.h>
#include "pcd8544.h"

//#define DEBUG  // à dé-commenter pour passer en mode debug (affichage moniteur série)
#define FIRMWARE "1.0.16"

#define PIN_LED_BACKLIGHT 6
#define PIN_RST 8  // Data/Command
#define PIN_DC 9
#define PIN_CS 10  // Chip select (SCE)
pcd8544 lcd(PIN_DC, PIN_RST, PIN_CS); // Create a pcd8544 object, hardware SPI will be used (sdin (MOSI) is on pin 11 and sclk on pin 13).

const int PIN_BUZZER = 4;
const int PIN_VOLTAGE = A0;

unsigned long timeLoop = 0;

const int pressureCount = 50;        // nb d'échantillons pour la moyenne glissante
int pressureIndice = 0;              // l'indice de l'échantillon courant pour le calcul de la moyenne glissante
float pressureTable[pressureCount];  // tableau pour stocker les valeurs permettant de calculer la moyenne glissante
float pressureTotal = 0;             // total pour le calcul de la moyenne glissante

const int sinkRateCount = 50;        // nb d'échantillons pour la moyenne glissante
int sinkRateIndice = 0;              // l'indice de l'échantillon courant pour le calcul de la moyenne glissante
float sinkRateTable[sinkRateCount];  // tableau pour stocker les valeurs permettant de calculer la moyenne glissante
float sinkRateTotal = 0;             // total pour le calcul de la moyenne glissante

int toneDelay = 0;

unsigned int calibrationData[7];
//unsigned long time = 0;
unsigned long timePrintSerialMonitor = 0;
unsigned long timePrintSerialMonitorOld = 0;
unsigned long timeSinkRate = 0;
unsigned long timeSinkRateOld = 0;
unsigned long timePrintDisplayVoltagePercentage = 0;
unsigned long timePrintDisplayPressure = 0;
unsigned long timePrintDisplayTemperature = 0;
/*
  unsigned long timeToneDelay = 0;
  unsigned long timeToneDelayOld = 0;
  unsigned long timeNoToneDelay = 0;
  unsigned long timeNoToneDelayOld = 0;
  unsigned long timePrintDisplaySinkRateMax = 0;
  unsigned long timePrintDisplaySinkRateMin = 0;
  unsigned long timePrintDisplayAltitudeAbsolueMax = 0;
  unsigned long timePrintDisplayAltitudeRelativeMax = 0;
*/

// Tension batterie
unsigned long timeVoltage = 0;
unsigned long timeVoltageOld = 0;
float voltage = 0;
float  voltagePercentage = 0;
int voltageCount = 0;

// print display
unsigned long timePrintDisplay = 0;
unsigned long timePrintDisplayOld = 0;
int X_DisplayBegin = 5;
int printDisplayScreen = 0;

// Temps de vol
boolean flight = false;
unsigned long flightStartMillis = 0;
unsigned long flightTotalSeconds = 0;
byte flightHours = 0;
int flightSecondsRemaining = 0;
byte flightMinutes = 0;
byte flightMinutesOld = 0;
byte flightSeconds = 0;
char flightTime[6] = "00:00";

// heure
char HoursMinutes[6];
byte minutesOld;

float TEMP = 0;  // 25°C pour 21 réels => recalage nécesaire ?
//float temperatureOffset = -3; // pour correction calage capteur
int tempInteger = 0;
int tempIntegerOld = 0;
float toneFreq = 0;
float toneFreqLowpass = 0;
float pressure = 0;  // 1020 hPa pour 1025 réels => recalage nécesaire ?
//float pressureOffset = 0; // pour correction calage capteur
float p0 = 0;
float lowpassFast = 0;
float lowpassSlow = 0;
float altitudeAbsolue = 0;  // v1.0.11
int altitudeAbsolueInteger;  // cast de altitudeAbsolue en Integer
int altitudeAbsolueMax = -1000;  // v1.0.11
float altitudeAbsolueOld = 0;  // v1.0.11
float altitudeRelative = 0;  // v1.0.11
int altitudeRelativeInteger;  // cast de altitudeRelative en Integer
int altitudeRelativeMax = 0;  // v1.0.11
float sinkRate = 0;
float sinkRateMin = 0;
float sinkRateMax = 0;
int ddsAcc;

void setup() {
#ifdef DEBUG
  Serial.begin(115200);
#endif

  //rtc.adjust(DateTime(F(__DATE__), F(__TIME__)));  // sets the RTC to the date & time this sketch was compiled

  pinMode(PIN_BUZZER, OUTPUT);
  pinMode(PIN_LED_BACKLIGHT, OUTPUT);
  digitalWrite(PIN_LED_BACKLIGHT, LOW);  // backlight, non utilisé
  lcd.begin();  // Always call lcd.begin() first.
  lcd.clear();
  pinMode(PIN_BUZZER, OUTPUT);
  Wire.begin();
  setupSensor();

  while (pressureIndice < pressureCount) {    // On remplit le tableau d'échantillons de pressions
    pressureTable[pressureIndice] = getPressure() / 100.0;
    pressureTotal = pressureTotal + pressureTable[pressureIndice];
    pressureIndice++;
  }
  pressure = pressureTotal / pressureCount;
  lowpassFast = lowpassSlow = p0 = pressure;
  altitudeAbsolue = altitudeAbsolueOld = 288.15 / 0.0065 * (1 - pow((pressure / 1013.25), (8.3144621 * 0.0065 / (0.02896 * 9.807))));
  pressureIndice = 0;

  for (sinkRateIndice = 0; sinkRateIndice < sinkRateCount; sinkRateIndice++) {    // On remplit le tableau d'échantillons avec des 0
    sinkRateTable[sinkRateIndice] = 0;
  }
  sinkRateIndice = 0;

  // Affichage version firmware
  lcd.setCursor(0, 2);
  lcd.print("   EBvario");
  lcd.setCursor(0, 4);
  lcd.print("   v");
  lcd.print(FIRMWARE);
  delay(2000);
  lcd.clear();
}

void loop() {
  moyenneGlissantePression();
  altitudeAbsolue = 288.15 / 0.0065 * (1 - pow((pressure / 1013.25), (8.3144621 * 0.0065 / (0.02896 * 9.807))));
  altitudeRelative = 288.15 / 0.0065 * (1 - pow((pressure / p0), (8.3144621 * 0.0065 / (0.02896 * 9.807))));
  moyenneGlissanteSinkRate();
  takeoffDetection();  // déclenche le début du temps de vol et le bip du vario
  memorisationMinMax();  // mémorisation des altitudes max et des taux de chute min/max
  printDisplay();
#ifdef DEBUG
  affichageMoniteurSerie();  // pour le débug
  Serial.println(millis() - timeLoop);
  timeLoop = millis();
#endif
}

long getPressure() {
  long D1, D2, dT, P;
  int64_t OFF, SENS;
  D1 = getData(0x48, 10);
  D2 = getData(0x50, 1);

  dT = D2 - ((long)calibrationData[5] << 8);
  TEMP = (2000 + (((int64_t)dT * (int64_t)calibrationData[6]) >> 23)) / (float)100;
  OFF = ((unsigned long)calibrationData[2] << 16) + (((int64_t)calibrationData[4] * dT) >> 7);
  SENS = ((unsigned long)calibrationData[1] << 15) + (((int64_t)calibrationData[3] * dT) >> 8);
  P = (((D1 * SENS) >> 21) - OFF) >> 15;

  return P;
}

long getData(byte command, byte del) {
  long result = 0;
  twiSendCommand(0x77, command);
  delay(del);
  twiSendCommand(0x77, 0x00);
  Wire.requestFrom(0x77, 3);
  if (Wire.available() != 3) {
#ifdef DEBUG
    Serial.println("Debug");
#endif
    Serial.println("Error: raw data not available");
  }
  for (int i = 0; i <= 2; i++) {
    result = (result << 8) | Wire.read();
  }
  return result;
}

void setupSensor() {
  twiSendCommand(0x77, 0x1e);
  delay(100);
  for (byte i = 1; i <= 6; i++) {
    unsigned int low, high;
    twiSendCommand(0x77, 0xa0 + i * 2);
    Wire.requestFrom(0x77, 2);
    if (Wire.available() != 2) Serial.println("Error: calibration data not available");
    high = Wire.read();
    low = Wire.read();
    calibrationData[i] = high << 8 | low;
#ifdef DEBUG
    Serial.print("calibration data #");
    Serial.print(i);
    Serial.print(" = ");
    Serial.println( calibrationData[i] );
#endif
  }
}

void twiSendCommand(byte address, byte command) {
  Wire.beginTransmission(address);
  if (!Wire.write(command)) {
#ifdef DEBUG
    Serial.println("Error: write()");
#endif
  }
  if (Wire.endTransmission()) {
#ifdef DEBUG
    Serial.print("Error when sending command: ");
    Serial.println(command, HEX);
#endif
  }
}

void moyenneGlissantePression() {  // moyenne glissante pour le calcul de pression
  pressureTotal = pressureTotal - pressureTable[pressureIndice];
  pressureTable[pressureIndice] = getPressure() / 100.0;
  pressureTotal = pressureTotal + pressureTable[pressureIndice];  // Ajout du dernier echantillon
  pressureIndice++;
  if (pressureIndice >= pressureCount) {   // si on est à la fin du tableau ...
    pressureIndice = 0;
  }
  pressure = pressureTotal / pressureCount;
}

void moyenneGlissanteSinkRate() {  // moyenne glissante pour le calcul de taux de chute
  timeSinkRate = millis();
  sinkRateTotal = sinkRateTotal - sinkRateTable[sinkRateIndice];
  sinkRateTable[sinkRateIndice] = ((altitudeAbsolue - altitudeAbsolueOld) / ((timeSinkRate - timeSinkRateOld) / 1000.0));
  sinkRateTotal = sinkRateTotal + sinkRateTable[sinkRateIndice];  // Ajout du dernier echantillon
  sinkRateIndice++;
  if (sinkRateIndice >= sinkRateCount) {   // si on est à la fin du tableau ...
    sinkRateIndice = 0;
  }
  altitudeAbsolueOld = altitudeAbsolue;
  timeSinkRateOld = timeSinkRate;
  sinkRate = sinkRateTotal / sinkRateCount;
}

void takeoffDetection() {
  calculBipVario();
  if (abs(altitudeRelative) >= 5 && flight == false) {
    flight = true;
#ifdef DEBUG
    Serial.println("takeoff !!!!");
#endif
  }
  if (flight == true) {
    gestionBipVario();  // gestion du bip de montée / descente du variomètre
    if (flightStartMillis == 0) {
      flightStartMillis = millis();
    }
    flightTotalSeconds = (millis() - flightStartMillis) / 1000;
    flightHours = flightTotalSeconds / 3600;
    flightSecondsRemaining = flightTotalSeconds % 3600;
    flightMinutes = flightSecondsRemaining / 60;
    //flightSeconds = flightSecondsRemaining % 60;
    sprintf(flightTime, "%02dh%02d", flightHours, flightMinutes);  // fills flightTime with text : %02d => instruction to convert a number (int variable or smaller) into text. %02d means "Use at least two digits. If only one digit, then put a zero at the left to make it two digits." If you don't want the zero, then use %d instead of %02d. If you want more zeros (like on a car odometer), then use %03d or %04d or whatever.
  }
}

void calculBipVario() {
  lowpassFast = lowpassFast + (pressure - lowpassFast) * 0.1;
  lowpassSlow = lowpassSlow + (pressure - lowpassSlow) * 0.05;
  toneFreq = (lowpassSlow - lowpassFast) * 5000;  // initial : 50
  toneFreqLowpass = toneFreqLowpass + (toneFreq - toneFreqLowpass) * 0.1;
  toneFreq = constrain(toneFreqLowpass, -500, 500);
  ddsAcc += toneFreq * 10 + 1500;  // initial : ddsAcc += toneFreq * 100 + 2000;
}

void gestionBipVario() { // gestion du bip de montée / descente du variomètre
  if (sinkRate < -1.0 || sinkRate > 0.1) {
    if (toneFreq < 0 || ddsAcc > 0) {
      tone(PIN_BUZZER, toneFreq + 510);
    }
    else {
      noTone(PIN_BUZZER);
    }
  }
  else {
    noTone(PIN_BUZZER);
  }
}

void memorisationMinMax() {  // mémorisation des altitudes max et des taux de chute min/max
  if (flight == true) {
    if (sinkRate < sinkRateMin) {
      sinkRateMin = sinkRate;
    }
    if (sinkRate > sinkRateMax) {
      sinkRateMax = sinkRate;
    }
    if (altitudeAbsolue > altitudeAbsolueMax) {
      altitudeAbsolueMax = altitudeAbsolue;
    }
    if (altitudeRelative > altitudeRelativeMax) {
      altitudeRelativeMax = altitudeRelative;
    }
  }
}

void readVoltage() {
  timeVoltage = millis();
  if (timeVoltage - timeVoltageOld > 1000) {
    timeVoltageOld = timeVoltage;
    /*
      voltage = 0;
      for (voltageCount = 0; voltageCount <= 9; voltageCount++) {   // moyenne sur 10 valeurs
      voltage = voltage + analogRead(PIN_VOLTAGE) * 3.3 / 1024.0 * 2;  // 3.3V pour une valeur de 1024 / 2 (diviseur de tension avec ratio 2)
      }
      voltage = voltage / 10.0;
    */
    voltage = analogRead(PIN_VOLTAGE) * 3.3 / 1024.0 * 2;
    voltagePercentage = (voltage - 3.4) * 100 / (4.1 - 3.4);  // correctif v1.0.12 : batterie vide (0%) si tension <= 3.4V; batterie pleine (100%) si tension >= 4,1V
    voltagePercentage = constrain(voltagePercentage, 0, 100);
  }
}

void printDisplay() {
  timePrintDisplay = millis();
  if (printDisplayScreen == 12) {
    printDisplayScreen = 0;
  }
  if (timePrintDisplay - timePrintDisplayOld >= 500) {
    printDisplayScreen ++;
#ifdef DEBUG
    Serial.println();
    Serial.println(printDisplayScreen);
#endif
    printDisplaySinkRate();
    printDisplayAltitudeRelative();
    if (printDisplayScreen == 1) {
      lcd.setCursor(4, 3);
      lcd.clearRestOfLine();
    }
    if (printDisplayScreen == 8) {
      printDisplayAltitudeRelativeMax();
    }
    /*
      printDisplayAltitudeAbsolue(); // suppression en v1.0.16
      if (printDisplayScreen == 8) {
      printDisplayAltitudeAbsolueMax();
      }
    */
    printDisplayHoursMinutes();
    printDisplayFlightTime();
    printDisplayVoltagePercentage();
    //printDisplayTemperature();
    //printDisplayPressure();
    /*
        Serial.print("PrintDisplay ");
        Serial.println(millis() - timePrintDisplay);
    */
    timePrintDisplayOld = timePrintDisplay;
  }
}

void printDisplaySinkRateMax() {
  if (flight == true) {
    lcd.print(sinkRateMax, 1);
    lcd.print("   ");
#ifdef DEBUG
    Serial.println("printDisplaySinkRateMax");
#endif
  }
}

void printDisplaySinkRate() {
  lcd.setCursor(0, 0);
  if (sinkRate >= 0) {
    lcd.print(" ");
  }
  lcd.print(sinkRate, 1);
  //lcd.print(" m/s    ");  // A sup X_DisplayBegin
  lcd.print(" m/s ");
  if (printDisplayScreen == 1) {
    lcd.print("      ");
  }
  if (printDisplayScreen == 4) {
    printDisplaySinkRateMin();
  }
  if (printDisplayScreen == 8) {
    lcd.print(" ");
    printDisplaySinkRateMax();
  }
#ifdef DEBUG
  Serial.println("printDisplaySinkRate");
#endif
}

void printDisplaySinkRateMin() {
  if (flight == true) {
    lcd.print(sinkRateMin, 1);
    lcd.print("   ");
#ifdef DEBUG
    Serial.println("printDisplaySinkRateMin");
#endif
  }
}

void printDisplayAltitudeRelative() {
  altitudeRelativeInteger = (int)(altitudeRelative);  // pour éviter les erreurs d'affichage dues à des arrondis
  lcd.setCursor(0, 2);
  if (altitudeRelativeInteger >= 0) {
    lcd.print(" ");
  }
  lcd.print(altitudeRelativeInteger);
  lcd.print(" m rel    ");
#ifdef DEBUG
  Serial.println("printDisplayAltitudeRelative");
#endif
}

void printDisplayAltitudeRelativeMax() {
  if (flight == true) {
    lcd.setCursor(4, 3);
    lcd.print(altitudeRelativeMax);
    lcd.print(" max");
#ifdef DEBUG
    Serial.println("printDisplayAltitudeRelativeMax");
#endif
  }
}

void printDisplayAltitudeAbsolue() {
  altitudeAbsolueInteger = (int)(altitudeAbsolue);  // pour éviter les erreurs d'affichage dues à des arrondis
  lcd.setCursor(0, 1);
  lcd.print(altitudeAbsolueInteger);
  lcd.print(" m abs    ");
#ifdef DEBUG
  Serial.println("printDisplayAltitudeAbsolue");
#endif
}

void printDisplayAltitudeAbsolueMax() {
  if (flight == true) {
    lcd.setCursor(4, 3);
    lcd.print(altitudeAbsolueMax);
    lcd.print(" abs");
#ifdef DEBUG
    Serial.println("printDisplayAltitudeAbsolueMax");
#endif
  }
}

void printDisplayHoursMinutes() {
  DateTime now = rtc.now();  // variable (now) à définir absolument dans une fonction. Sinon, ca ne plait pas au compilateur car cette variable ne sera pas définie dans les autres cases.
  sprintf(HoursMinutes, "%02dh%02d", now.hour(), now.minute());  // fills flightTime with text : %02d => instruction to convert a number (int variable or smaller) into text. %02d means "Use at least two digits. If only one digit, then put a zero at the left to make it two digits." If you don't want the zero, then use %d instead of %02d. If you want more zeros (like on a car odometer), then use %03d or %04d or whatever.
  if (HoursMinutes[4] != minutesOld) {
    minutesOld = HoursMinutes[4];
    lcd.setCursor(1, 4);
    lcd.print(HoursMinutes);
#ifdef DEBUG
    Serial.println("printDisplayHoursMinutes");
#endif
  }
}

void printDisplayFlightTime() {
  if (flight == true && flightTime[4] != flightMinutesOld) {
    flightMinutesOld = flightTime[4];
    lcd.setCursor(9, 4);
    lcd.print(flightTime);
#ifdef DEBUG
    Serial.println("printDisplayFlightTime");
#endif
  }
}

void printDisplayVoltagePercentage() {
  if (millis() - timePrintDisplayVoltagePercentage >= 5000) {  // Update affichage toutes les 5 secondes
    readVoltage();
    lcd.setCursor(1, 5);
    lcd.print(voltagePercentage, 0);
    lcd.print(" %  ");
    //lcd.print(voltage);  // pas d'affichage en 1.0.16
    //lcd.print(" V ");    // pas d'affichage en 1.0.16
#ifdef DEBUG
    Serial.println("printDisplayVoltagePercentage");
#endif
  }
}

void printDisplayTemperature() {
  if (millis() - timePrintDisplayTemperature >= 5000) {  // Update affichage toutes les 5 secondes
    lcd.setCursor(0, 4);
    lcd.print(TEMP, 0);
    lcd.print(" C    ");
#ifdef DEBUG
    Serial.println("printDisplayTemperature");
#endif
  }
}

void printDisplayPressure() {
  if (millis() - timePrintDisplayPressure >= 5000) {  // Update affichage toutes les 5 secondes
    lcd.setCursor(0, 5);
    lcd.print(pressure, 0);
    lcd.print(" hPa    ");
#ifdef DEBUG
    Serial.println("printDisplayPressure");
#endif
  }
}

void affichageMoniteurSerie() {  // 72ms pour l'affichage de l'ensemble sur le moniteur série
#ifdef DEBUG
  timePrintSerialMonitor = millis();
  if (timePrintSerialMonitor - timePrintSerialMonitorOld >= 1000) {
    DateTime now = rtc.now();
    sprintf(HoursMinutes, "%02dh%02d", now.hour(), now.minute());  // fills flightTime with text : %02d => instruction to convert a number (int variable or smaller) into text. %02d means "Use at least two digits. If only one digit, then put a zero at the left to make it two digits." If you don't want the zero, then use %d instead of %02d. If you want more zeros (like on a car odometer), then use %03d or %04d or whatever.
    if (HoursMinutes[4] != minutesOld) {
      minutesOld = HoursMinutes[4];
      Serial.print(HoursMinutes);
    }
    Serial.print("\t");
    if (flightTime[4] != flightMinutesOld) {
      flightMinutesOld = flightTime[4];
      Serial.print(flightTime);
    }
    Serial.print("\t");
    Serial.print(TEMP, 1);
    Serial.print(" C   ");
    Serial.print(pressure, 1);
    Serial.print(" hPa   ");
    Serial.print(altitudeAbsolueInteger);
    Serial.print(" m abs   ");
    Serial.print(altitudeAbsolueMax);
    Serial.print(" m abs max   ");
    Serial.print(altitudeRelativeInteger);
    Serial.print(" m rel   ");
    Serial.print(altitudeRelativeMax);
    Serial.print(" m rel max   ");
    Serial.print(sinkRate, 1);
    Serial.print(" m/s   ");
    Serial.print(sinkRateMin, 1);
    Serial.print(" m/s min   ");
    Serial.print(sinkRateMax, 1);
    Serial.print(" m/s max   ");
    readVoltage();
    Serial.print("Batterie ");
    Serial.print(voltagePercentage, 0);
    Serial.print("%   ");
    Serial.print("timePrint ");
    Serial.print(millis() - timePrintSerialMonitor);
    Serial.println(" ms");
    timePrintSerialMonitorOld = timePrintSerialMonitor;
  }
#endif
}
